/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/
namespace IRDB_SDK
{
	using namespace std;

	// enums
	using ADFileType_t =  enum ADFileType
		{ 
		 adftELFEXE, 
		 adftELFSO, 
		 adftCGC, 
		 adftPE, 
		 adftNone 
		};

	using ADMachineType_t = enum ADMachineType
		{ 
		 admtAarch64,  
		 admtX86_64, 
		 admtI386, 
		 admtNone 
		};

	using DatabaseErrorType_t = enum DatabaseErrorType
    		{
		 detVariantNotInDatabase, 
		 detVariantTableNotRegistered
		};

	using ICFSAnalysisStatus_t = enum ICFSAnalysisStatus
		{ 
		 iasAnalysisIncomplete, 
		 iasAnalysisModuleComplete, 
		 iasAnalysisComplete 
		};

	using IRDBType_t = enum IRDBType
		{
		 itUnknown,
		 itNumeric,
		 itPointer,
		 itVoid,
		 itVariadic,
		 itInt,
		 itChar,
		 itFloat,
		 itDouble,
		 itTypedef,
		 itSubtype,
		 itFunc,
		 itAggregate
		};

	// enum renames
	//using ADFileType_t         = enum ADFileType;
	//using ADMachineType_t      = enum ADMachineType;
	//using DatabaseErrorType_t  = enum DatabaseErrorType;
	//using ICFSAnalysisStatus_t = enum ICFSAnalysisStatus; 
	//using IRDBType_t           = enum IRDBType;



	// forward decls;
	class AddressID_t;
	class ArchitectureDescription_t;
	class BaseObj_t;
	class DatabaseError_t;
	class DBinterface_t;
	class DecodedInstruction_t;
	class DecodedOperand_t;
	class Doip_t;
	class EhProgram_t;
	class EhCallSite_t;
	class File_t;
	class FileIR_t;
	class Function_t;
	class ICFS_t;
	class Instruction_t;
	class IRDBObjects_t;
	class pqxxDB_t;
	class Relocation_t;
	class DataScoop_t;
	class Type_t;
	class BasicType_t;
	class PointerType_t;
	class AggregateType_t;
	class FuncType_t;
	class VariantID_t;

	// typedefs
	using VirtualOffset_t         = uintptr_t;
	using DatabaseID_t            = int32_t;
	using SchemaVersionID_t       = int32_t;
	using EhProgramInstruction_t  = string;
	using EhProgramListing_t      = vector<EhProgramInstruction_t>;
	using EhProgramSet_t          = set<EhProgram_t*>;
	using TTOrderVector_t         = vector<int>;
	using EhCallSiteSet_t         = set<EhCallSite_t*>;
	using FunctionSet_t           = set<Function_t*>;
	using AddressSet_t            = set<AddressID_t*>;
	using ICFSSet_t               = set<ICFS_t*>;
	using RelocationSet_t         = set<Relocation_t*>; 
	using InstructionSet_t        = set<Instruction_t*>; 
	using DataScoopSet_t          = set<DataScoop_t*>;
	using TypeSet_t               = set<Type_t*>;
	using TypeVector_t            = vector<Type_t*>;
	using FileSet_t               = set<File_t*>;
	using DecodedOperandVector_t  = vector<shared_ptr < DecodedOperand_t > >;



}
