/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

namespace IRDB_SDK
{

	using namespace std;

	class FileIR_t : virtual public BaseObj_t
	{
		protected:
			FileIR_t(){}
			FileIR_t(const FileIR_t& p_copy) = delete;
		public:
			virtual ~FileIR_t() {}
		  
			// accessors 
			virtual const FunctionSet_t&    getFunctions()      const = 0;
			virtual const InstructionSet_t& getInstructions()   const = 0;
			virtual const AddressSet_t&     getAddresses()      const = 0;
			virtual const RelocationSet_t&  getRelocations()    const = 0;
			virtual const DataScoopSet_t&   getDataScoops()     const = 0;
			virtual const ICFSSet_t&        getAllICFS()        const = 0;
			virtual const EhProgramSet_t&   getAllEhPrograms()  const = 0;
			virtual const EhCallSiteSet_t&  getAllEhCallSites() const = 0;
			virtual File_t*                 getFile()           const = 0;
			virtual DatabaseID_t            getMaxBaseID()      const = 0;
			virtual DataScoop_t*            findScoop(
			         const VirtualOffset_t &p_addr)             const = 0;

			virtual void   setAllEhPrograms(const EhProgramSet_t& new_pgms)               = 0;
			virtual void   setBaseIDS()                                                   = 0;
			virtual void   assembleRegistry()                                             = 0;
			virtual void   registerAssembly(Instruction_t *p_instr, string p_assembly)    = 0;
			virtual void   unregisterAssembly(Instruction_t *p_instr)                     = 0;
			virtual string lookupAssembly(Instruction_t *p_instr)                         = 0;
			virtual void   changeRegistryKey(
					             Instruction_t* p_orig, Instruction_t* p_updated) = 0;
			virtual void   splitScoop(
						DataScoop_t *p_to_split, 
						const VirtualOffset_t &p_addr, 
						size_t p_size, 
						DataScoop_t* &p_before,
						DataScoop_t* &p_containing, 
						DataScoop_t* &p_after, 
						DatabaseID_t *p_max_id=NULL
					       )                                                    = 0;

			virtual void writeToDB(std::ostream *verbose_logging=&std::cerr) = 0;

			virtual EhCallSite_t* addEhCallSite_t(Instruction_t* for_insn, const uint64_t enc=0, Instruction_t* lp=nullptr) = 0;
			virtual Relocation_t* addNewRelocation(
					BaseObj_t* from_obj, 
					int32_t _offset, 
					string _type, 
					BaseObj_t* p_wrt_obj=nullptr, 
					int32_t p_addend=0
					) = 0;

			virtual EhProgram_t* addEhProgram(
					Instruction_t* insn=nullptr, 
					const uint64_t caf=1, 
					const int64_t daf=1, 
					const uint8_t rr=1, 
					const uint8_t p_ptrsize=8,
					const EhProgramListing_t& p_cie_program={},
					const EhProgramListing_t& p_fde_program={}
					) = 0;

			virtual AddressID_t* addNewAddress(const DatabaseID_t& myfileID, const VirtualOffset_t& voff=0) =0 ;
			virtual ICFS_t*      addNewICFS   (Instruction_t* insn=nullptr, const InstructionSet_t& targets={}, const ICFSAnalysisStatus_t& status=iasAnalysisIncomplete) = 0; 

			virtual Instruction_t* addNewInstruction(
					AddressID_t* addr=nullptr, 
					Function_t* func=nullptr, 
					const string& bits="", 
					const string& comment="user-added", 
					AddressID_t* indTarg=nullptr
					) =0;
			virtual DataScoop_t* addNewDataScoop(    
					const string& p_name="user-added",
					AddressID_t* p_start=nullptr,
					AddressID_t* p_end=nullptr,
					Type_t* p_type=nullptr,
					uint8_t p_permissions=0,
					bool p_is_relro=false,
					const string &p_contents="",
					DatabaseID_t id=BaseObj_t::NOT_IN_DATABASE
					) = 0;



			virtual void removeScoop(DataScoop_t* s) = 0;
			virtual void moveRelocation(Relocation_t* reloc, Instruction_t* from, Instruction_t* to) = 0;
			virtual EhProgram_t* copyEhProgram(const EhProgram_t& orig) = 0;


		// static methods
			static uint32_t  getArchitectureBitWidth() ;
			static const IRDB_SDK::ArchitectureDescription_t* getArchitecture() ;


		// factories:
			static unique_ptr<FileIR_t> factory(VariantID_t *p_progid, File_t* p_fid=nullptr);


	};

}
